import requests
import subprocess, shlex
import argparse
import os
import logging
import urllib3

urllib3.disable_warnings()

def main():
    parser = argparse.ArgumentParser(description='Required args for recursive clone')
    parser.add_argument('--group_id', metavar='group_id', type=int,
                        help='Id of a group in gitlab', required=True)

    parser.add_argument('--branch', metavar='branch', type=str,
                        help='Branch to clone in all repos [by default master]', default='master')

    parser.add_argument('--http', help='Clone via http instead of ssh', action='store_true')

    parser.add_argument(
        '--gitlab-url',
        metavar='gitlab',
        type=str,
        default=os.environ.get('GITLAB_URL', "gitlab.com"),
        help='Gitlab address [by default gitlab.com]')

    parser.add_argument(
        '--token',
        metavar='token',
        type=str,
        default=os.environ.get('GITLAB_TOKEN'),
        help='Gitlab Token')

    args = parser.parse_args()
    request_param = args.__dict__

    clone(**request_param)


def clone(group_id, branch, token, gitlab_url, http):
    total_pages = 1
    page = 0
    while page < total_pages:
        page += 1
        response = requests.get(
            f"{gitlab_url}/api/v4/groups/{group_id}/projects?private_token={token}&include_subgroups=True&per_page=100&page={page}&with_shared=False", verify=False)
        for project in response.json():
            path = project['path_with_namespace']
            if http:
                url_to_repo = project[f'http_url_to_repo'].replace("//",f"//token:{token}@")
            else:
                url_to_repo = project[f'ssh_url_to_repo']
            try:
                if not os.path.exists(path):
                    c=f"git clone --branch {branch} {url_to_repo} {path}"
                    logging.info(f"command: {c}")
                    command = shlex.split(c)
                    result_code = subprocess.Popen(command)
                else:
                    logging.info(f"{path} already exists")
                    command = shlex.split(f"cd {path}; git pull  {path}; cd -")
                    result_code = subprocess.Popen(command)

            except Exception as e:
                logging.error(f"Error on {ssh_url_to_repo}: {e}")

        total_pages = int(response.headers['X-Total-Pages'])


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

requirements = ['requests']

test_requirements = ['pytest', ]

setup(
    author="Arseniy Antonov",
    author_email='arseny.antonov@gmail.com',
    classifiers=[
        'Framework :: Pytest',
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    description="Gitlab tool for recursive clone",
    install_requires=requirements,
    license="MIT license",
    long_description=readme + '\n\n',
    include_package_data=True,
    keywords=[
        'gitlab', 'gitlab-api',
    ],
    name='gitlab-clone',
    packages=find_packages(include=['gitlab_clone']),
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/ArseniyAntonov/gitlab-group-clone',
    version='1.2.0',
    entry_points={
        "console_scripts": [
            "gitlab-clone=gitlab_clone.clonner:main",
        ]
    },
    zip_safe=False,
)
